package com.css.simulation.resource.scene.service;

import api.common.pojo.param.scene.ScenePackageParam;
import api.common.pojo.po.scene.ScenePackagePO;
import api.common.pojo.po.scene.ScenePackageSublistPO;
import api.common.pojo.vo.scene.ScenePackageSublistVO;
import api.common.util.StringUtil;

import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.scene.mapper.ScenePackageMapper;
import com.css.simulation.resource.scene.mapper.ScenePackageSublistMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Service
public class ScenePackageService {

    @Resource
    private ScenePackageMapper scenePackageMapper;
    @Resource
    private ScenePackageSublistMapper scenePackageSublistMapper;




    /**
     * 测试包数据录入：
     */
    @SneakyThrows
    public void saveScenePackage(ScenePackagePO params){
        params.setCreateTime(TimeUtil.getNowForMysql());
        params.setModifyTime(TimeUtil.getNowForMysql());
        params.setCreateUserId(AuthUtil.getCurrentUserId());
        params.setPackageId(StringUtil.getRandomUUID());
        params.setIsDeleted("0");
        scenePackageMapper.saveScenePackage(params);
    }
    //查询测试包主表列表
    @SneakyThrows
    public   List<ScenePackagePO> queryScenePackageList(ScenePackageParam params){
        List<ScenePackagePO> list = scenePackageMapper.queryScenePackageList(params);
        return list;
    }
    //查询测试包主表一条数据详情
    @SneakyThrows
    public   ScenePackagePO queryScenePackage(ScenePackageParam params){

        return  scenePackageMapper.queryScenePackage(params);
    }
    //修改主表数据
    @SneakyThrows
    public   void updateScenePackage(ScenePackagePO params) {
        params.setModifyUserId(StringUtil.getRandomUUID());
        params.setModifyTime(TimeUtil.getNowForMysql());
        scenePackageMapper.updateScenePackage(params);
    }

    //查询一个测试包的指标列表
    @SneakyThrows
    public ScenePackageSublistVO queryScenePackageSublistList(ScenePackageParam params){
        ScenePackageSublistPO po=new ScenePackageSublistPO();
        po.setParentId(params.getPackageId());
        ScenePackageSublistVO vo= scenePackageMapper.queryScenePackageBySublist(params.getPackageId());
        List<ScenePackageSublistVO> list =selectScenePackageSubListTree(null,true,params.getPackageId());

        vo.setList(list);
        return vo;
    }

    /**
     * 级联获取场景指标
     * @param parentVoList 首次传null
     * @param isRoot 首次传true
     * @param scenePackageId 场景包id
     * @return
     */
    public List<ScenePackageSublistVO> selectScenePackageSubListTree(List<ScenePackageSublistVO> parentVoList,
                                                                     boolean isRoot,String scenePackageId){
        ScenePackageSublistPO params=new ScenePackageSublistPO();
        params.setParentId(scenePackageId);
        if(isRoot){
            //查找一级节点
            parentVoList = scenePackageSublistMapper.queryScenePackageSublistList(params);
            if(parentVoList != null && parentVoList.size() > 0){
                selectScenePackageSubListTree(parentVoList, false, null);
            }
        }else{
            //获取子节点集合
            for(ScenePackageSublistVO pvo : parentVoList){
                params.setParentId(pvo.getSublistId());
                List<ScenePackageSublistVO> cvoList = scenePackageSublistMapper.queryScenePackageSublistList(params);
                if(cvoList != null && cvoList.size() > 0){
                    //存入父节点集合中
                    pvo.setList(cvoList);
                    //继续查找下一节点
                    selectScenePackageSubListTree(cvoList,false, null);
                }
            }
        }
        return parentVoList;
    }



}
