package com.css.simulation.resource.scene.service;

import api.common.pojo.param.scene.StandardsRegulationsParam;
import api.common.pojo.po.scene.StandardsRegulationsPO;
import api.common.util.FileUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.scene.mapper.StandardsRegulationsMapper;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class StandardsRegulationsService {

    @Resource
    private StandardsRegulationsMapper standardsRegulationsMapper;



    /**
     * 获取标准法规场景数据：
     */
    @SneakyThrows
    public void importBzFg(StandardsRegulationsParam param) {
        //1 获取所有目录下所有场景
     //  String dataUrl0 ="C:\\Users\\CSS\\Desktop\\GQ\\仿真云平台\\场景数据样例\\标准法规场景\\ENCAP_001_left25%_stop30";     // 场景数据根路径
        String dataUrl0=param.getPath();
        File[] scenes = FileUtil.getDirectory(dataUrl0).listFiles();
        if (scenes == null) {
            throw new RuntimeException("目录：" + dataUrl0 + " 下没有文件！");
        }

        String fileName = null;
        try {

            for (File scene : scenes) {
                String dataUrl = scene.getAbsolutePath();
                if(dataUrl.indexOf(".json")!=-1){
                    //获取文件名
                    File tempFile =new File( dataUrl.trim());
                    fileName = tempFile.getName();
                    fileName=  fileName.substring(0,fileName.lastIndexOf("."));
                // -------------------------------- 读取结构化数据文件 --------------------------------
                List<String> exceptionList = new ArrayList<>();     // 异常列表
                // -------------------------------- label.json --------------------------------
                String json = FileUtil.readFile(dataUrl);
                    ObjectMapper objMap = new ObjectMapper();
                    JsonNode root = objMap.readTree(json);

                    StandardsRegulationsPO standardsRegulationsPO = null;
                    standardsRegulationsPO = standardsRegulationsPO.builder()
                        .regulationsId(StringUtil.getRandomUUID())
                        .sceneName(root.path("场景名称").asText())
                        .regulationType(root.path("法规类型").asText())
                        .standardType(root.path("标准类型").asText())
                        .isDeleted("0")
                        .build();
                // -------------------------------- 保存数据到 mysql --------------------------------
                    standardsRegulationsPO.setCreateTime(TimeUtil.getNowForMysql());
                    standardsRegulationsPO.setModifyTime(TimeUtil.getNowForMysql());
                    standardsRegulationsPO.setCreateUserId(AuthUtil.getCurrentUserId());
                    if(null!=param.getJsonAddress()&&!param.getJsonAddress().equals("")){
                        standardsRegulationsPO.setJsonAddress(param.getJsonAddress());
                    }
                    if(null!=param.getXmlAddress()&&!param.getXmlAddress().equals("")){
                        standardsRegulationsPO.setXmlAddress(param.getXmlAddress());
                    }
                    if(null!=param.getXodrAddress()&&!param.getXodrAddress().equals("")){
                        standardsRegulationsPO.setXodrAddress(param.getXodrAddress());
                    }
                    if(null!=param.getOsgbAddress()&&!param.getOsgbAddress().equals("")){
                        standardsRegulationsPO.setOsgbAddress(param.getOsgbAddress());
                    }
                    if(null!=param.getVideoAddress()&&!param.getVideoAddress().equals("")){
                        standardsRegulationsPO.setVideoAddress(param.getVideoAddress());
                    }
                    standardsRegulationsMapper.saveStandardsRegulations(standardsRegulationsPO);

            }

            }
        } catch (Exception e) {
            log.error("标准法规场景" + fileName + " 解析出错：", e);
        }


    }
//查看列表
    public   List<StandardsRegulationsPO> querySceneNaturalList(StandardsRegulationsParam params) {
        List<StandardsRegulationsPO> list = standardsRegulationsMapper.queryStandardsRegulationsList(params);
        return list;
    }
}
